@@ -5,7 +5,7 @@ class VaultStructBase(abc.ABC):
     @classmethod
     @abc.abstractmethod
     def build_from_vault_key(cls, vault_key, vault_value):
-        pass
+        raise NotImplementedError(f"{cls.build_from_vault_key.__name__} was not implemented in the subclass that inherited this.")
 
 
 class VaultStructDictBase(VaultStructBase, dict, abc.ABC):
